/*************************************************************************************************************
/ IEC60870 RTU
/ Copyright by Panasonic Electric Works Europe AG 2003-2013
/ RT 02.03.2012
**************************************************************************************************************/

//Global variables / parameters
var m_iNumberOfRecords = 5;		//History: point of records to load in one request
var m_iSizeOfRecord = 5;			//History: point of data per record
var m_iMaxLines = 98;				//Max lines
var m_iError = 0;

//Global variables for displaying the plc data
var m_sTime;
var m_sDate;
var m_sCode;
var m_sText;
var m_sNumber;

//System global 
var PLCreg = new Array(m_iNumberOfRecords*m_iSizeOfRecord);	//Response data array (16 bit words or integers)
var XML_Data;							//Buffer for MEWTOCOL request command
var PLC_StationAddress = "EE";	//PLC MEWTOCOL station address ("EE", "01"..."64")

//Global variables for interval handling
var m_bDownloadOK;					//Handshake for data download
var m_iNotFirstTime;					//Work different in the first cycle
var m_iLoopInterval;					//Checks the transfer status and start the next download of data
var m_iWait = 1;						//Wait before repeat loading data
var m_iDataLoop;						//Selection of the trend line
var m_iLines;
var m_iInterval = 500;					//Set update m_iInterval
var zeroes="00000";					//For generating leading zeroes

var m_LogError="DT31535";//Full PLC address of first m_LogError register
var m_LogErrorDT;						//First m_LogError address (DT area) calculated from m_LogError
var m_bIsAreaDT;						//m_LogError in DT or FL area


//JavaScript started on page load
window.onload = function()
{
	m_bDownloadOK = 1;			//Start download
	m_iNotFirstTime = 0;			//At first start update immediately, later in one step
	m_iLoopInterval = 0;			//Task manager
	m_iDataLoop = 0;				//Record selector
	m_iLines = m_iMaxLines-2;	//Max. lines to display
	ResetTags();					//Clear the message area

	m_LogErrorDT = parseInt(m_LogError.substring(2,m_LogError.length));	//String "DT90054" --> 90054
	m_bIsAreaDT = m_LogError.indexOf("DT");										// FL is -1
	XML_Data = GenerateRD(PLC_StationAddress, m_LogErrorDT, m_iNumberOfRecords*m_iSizeOfRecord, m_bIsAreaDT);	//%01#RDD0019900204** --> XML_Data

	m_iInterval = 500;
	intT = setInterval(function(){CheckIntervall();},m_iInterval);	//Set update interval

}

function CheckIntervall()
{
	$("ajaxDebug1").innerHTML = m_iNotFirstTime;

	m_iLoopInterval = m_iLoopInterval + 1;
	if (m_iLoopInterval >= m_iLines*m_iWait || m_bDownloadOK == 2)
	{
		m_iLoopInterval = 0;
		m_bDownloadOK = 1;	// Start again
		ResetTags();			//Clear the message area
	}
	if (m_bDownloadOK == 1)
	{
		if (m_iDataLoop < m_iLines)
		{
			m_bDownloadOK = 0;		// Reset flag to recognize errors
			XML_Data = GenerateRD(PLC_StationAddress, m_LogErrorDT + m_iDataLoop * m_iSizeOfRecord, m_iNumberOfRecords * m_iSizeOfRecord, m_bIsAreaDT); //%01#RDD0019900204** --> XML_Data
			UpdateData(XML_Data);	// Request new data
		}
		else
		{	// List completed
			m_iNotFirstTime = m_iNotFirstTime + 1;
			ResetTags();	//Clear the message area
		}
	}
}


// this function checks the status of the xhr object and calls the corresponding routines to
// display the variables
function DisplayPLCdata()
{
	var iIndex;
	var rxHor;
	var rxMin;
	var rxSec;
	var rxDay;
	var rxMon;
	var rxYar;
	var rxCode;

	if(xhr.readyState == XHR_READYSTATE_COMPLETED)
	{
		if (xhr.status == XHR_STATUS_OK) {
			try {
				if (XML_Data.indexOf(".xml") > 0)
					plc = eval("(" + xhr.responseText + ")");//Get PLC data
				else
					decodeRD_BCD(xhr.responseText); //RD response string, true=integer false=word
			}
			catch (e) {
				doError(e + "\nPLC or JSON read error.");
				return;
			}

			for (var iLoop = 0; iLoop < m_iNumberOfRecords; iLoop++) {
				iIndex = iLoop * m_iSizeOfRecord;
				rxHor = PLCreg[iIndex + 1].substring(2, 4);
				rxMin = PLCreg[iIndex + 0].substring(0, 2);
				rxSec = PLCreg[iIndex + 0].substring(2, 4);
				rxMon = PLCreg[iIndex + 1].substring(0, 2);
				rxDay = PLCreg[iIndex + 2].substring(2, 4);
				rxYar = PLCreg[iIndex + 2].substring(0, 2);
				rxCode = parseInt(PLCreg[iIndex + 3], 16);

				if (parseInt(rxYar)<70)
					rxYar = "20" + rxYar;
				else
					rxYar = "19" + rxYar;
					
				m_sTime = m_sTime + rxHor + ":" + rxMin + ":" + rxSec + "<br>&nbsp;";
				m_sDate = m_sDate + rxYar + "-" + rxMon + "-" + rxDay + "<br>&nbsp;";
				m_sCode = m_sCode + rxCode + "<br>&nbsp;";
				m_sNumber = m_sNumber + m_iDataLoop + "<br>&nbsp;";

				switch (rxCode) {
					case 0:
						m_sText = m_sText + ".<br>&nbsp;";
						break;
					case 1:
						m_sText = m_sText + "PLC: new start<br>&nbsp;";
						break;
					case 2:
						m_sText = m_sText + "PLC: online programming during runtime<br>&nbsp;";
						break;
					case 3:
						m_sText = m_sText + "IEC60870: buffer 1 restart<br>&nbsp;";
						break;
					case 4:
						m_sText = m_sText + "IEC60870: buffer 2 restart<br>&nbsp;";
						break;
					case 10:
						m_sText = m_sText + "PLC: hardware error<br>&nbsp;";
						break;
					case 11:
						m_sText = m_sText + "PLC: intelligent unit hw error<br>&nbsp;";
						break;
					case 12:
						m_sText = m_sText + "PLC: verify error I/O unit<br>&nbsp;";
						break;
					case 13:
						m_sText = m_sText + "PLC: constant cycle alarm<br>&nbsp;";
						break;
					case 42:
						m_sText = m_sText + "PLC: self diagnosis error<br>&nbsp;";
						break;
					case 45:
						m_sText = m_sText + "PLC: operation error<br>&nbsp;";
						break;
					case 50:
						m_sText = m_sText + "PLC: battery error<br>&nbsp;";
						break;
					case 101:
						m_sText = m_sText + "IEC60870: Backup in progress<br>&nbsp;";
						break;
					case 102:
						m_sText = m_sText + "IEC60870: Restore in progress<br>&nbsp;";
						break;
					case 103:
						m_sText = m_sText + "IEC60870: Backup successful<br>&nbsp;";
						break;
					case 104:
						m_sText = m_sText + "IEC60870: Restore successful<br>&nbsp;";
						break;
					case 148:
						m_sText = m_sText + "PLC: Reset in case of communication error<br>&nbsp;";
						break;
					case 232:
						m_sText = m_sText + "PLC: serial port error<br>&nbsp;";
						break;
					default:
						m_sText = m_sText + "Unknown error<br>&nbsp;";
						break;
				}
				m_iDataLoop = m_iDataLoop + 1;
			} //end for

			if (rxCode == 0)	// Stop in case of no more error messages
				m_iDataLoop = m_iLines+1;

			if (m_iNotFirstTime == 0 || m_iDataLoop >= m_iLines) {
				$("ajaxTime").innerHTML = m_sTime;
				$("ajaxDate").innerHTML = m_sDate;
				$("ajaxErrorNumber").innerHTML = m_sCode;
				$("ajaxMessage").innerHTML = m_sText;
				$("ajaxNumber").innerHTML = m_sNumber;
			}
			m_iError = 0; // read ready
			m_bDownloadOK = 1;
		}//end if XHR_STATUS_OK
		else
		{
			m_iError = m_iError + 1; // read error
			if (m_iError > 5) {
				m_iError = 0; // error reset
				alert("Message returned, but with error status.");
			}
			m_bDownloadOK = 2;
		}//end else
	}//end if XHR_READYSTATE_COMPLETED
}


//This function resets the list tags
function ResetTags()
{
	m_iDataLoop = 0;
	m_sTime = "";
	m_sDate = "";
	m_sCode = "";
	m_sText = "";
	m_sNumber = "";
}

/* End of file */